<?php
/*
 * Customizes frontend forms and corrects media queries
 */
defined( 'ABSPATH' ) || exit;

/**
 * Translates the title field label.
 *
 * @since 1.0.0
 * @since 1.0.3 Checks that the ACF field is found.
 *
 * @param array	$field The ACF field
 *
 * @return array The ACF field
 */
add_filter( 'acf/load_field/name=_post_title', 'it_acfforms_post_title_field' );
function it_acfforms_post_title_field( $field ) {
	$post_type_field = acf_get_field( 'it-acfforms-post-type' );
	if ( ! empty( $post_type_field ) ) {
		$field['label'] = apply_filters( 'it_acfforms_title_label', esc_html__( 'Title', 'it-acfforms' ), $post_type_field['value'] );
	}
	return $field;
}

/**
 * Translates the editor label and disable media upload.
 *
 * @since 1.0.0
 * @since 1.0.3 Checks that the ACF field is found.
 *
 * @param array	$field The ACF field
 *
 * @return array The ACF field
 */
add_filter( 'acf/load_field/name=_post_content', 'it_acfforms_post_editor_field' );
function it_acfforms_post_editor_field( $field ) {
	$post_type_field = acf_get_field( 'it-acfforms-post-type' );
	if ( ! empty( $post_type_field ) ) {
		$field['label'] = apply_filters( 'it_acfforms_editor_label', esc_html__( 'Content', 'it-acfforms' ), $post_type_field['value'] );
	}
	$field['media_upload'] = 0;
	return $field;
}

/**
 * Filters media request according to fields settings.
 *
 * - Gives the value of -1 to the parent if it’s 0.
 *   Useful in the case of new publications if the field's ACF option limits it
 *   to those uploaded to the publication. The media parent ID in this case
 *   defaults to 0, which returns all unattached files.
 * - Displays only the author's media (except for those with edit_pages cap).
 *
 * Not only affects the frontend form, but also the admin because the Media
 * Library is in wp-admin.
 *
 * @since 1.0.0
 */
add_filter( 'acf/load_field/type=image', 'it_acfforms_media_fields_query' );
add_filter( 'acf/load_field/type=gallery', 'it_acfforms_media_fields_query' );
add_filter( 'acf/load_field/type=file', 'it_acfforms_media_fields_query' );
function it_acfforms_media_fields_query( $field ) {
	if ( 'uploadedTo' == $field['library'] ) {
		add_filter( 'ajax_query_attachments_args', function ( $query ) {
			if ( 0 == $query['post_parent'] ) {
				$query['post_parent'] = -1;
			}
			return $query;
		} );
	}
	if ( 1 == $field['it_acfforms_limit_media'] ) {
		add_filter( 'ajax_query_attachments_args', function ( $query ) {
			if ( ! current_user_can( 'edit_pages' ) ) {
				$query['author'] = get_current_user_id();
			}
			return $query;
		} );
	}
	return $field;
}

/**
 * Returns the gallery field only if the user can upload_files.
 *
 * Otherwise, the add file link generates a JS error because wp.media isn’t
 * defined. By forcing the loading of wp.media for everyone, adding a file via
 * the media library returns an unauthorized access error.
 *
 * @since 1.0.0
 *
 * @param array	$field Le champ ACF
 *
 * @return array|bool The ACF field or false if the cap is missing.
 */
add_filter( 'acf/prepare_field/type=gallery', 'it_acfforms_gallery_field_cap' );
function it_acfforms_gallery_field_cap( $field ) {
	if ( current_user_can( 'upload_files' ) ) {
		return $field;
	}
	return false;
}

/**
 * Sets the post fields from submitted values.
 *
 * These values are only submitted on front form :
 * - _post_title and _post_content (acf supported post fields).
 * - it-acfforms-post-{post_field} fields (plugin default post fields).
 *
 * But Post Fields will be updated in admin if settings are set to select or
 * custom.
 *
 * Supported substitution variable :
 * - {key} for metadata, submitted or not, from ACF or not.
 * - %key% for some User or Post properties (see doc.)
 *
 * Acts before ACF registers the Post, so that the old value can be compared.
 *
 * @since 1.0.0
 *
 * @param int $post_id
 */
add_action( 'acf/save_post', 'it_acfforms_save_post_fields', 5 );
function it_acfforms_save_post_fields( $post_id ) {

	// Acts only if ACF data has been sent.
	if ( empty( $_POST['acf'] ) ) {
		return;
	}
    $new_values = $_POST['acf'];

	// Acts only if the Post exist.
	$post = get_post( $post_id );
	if ( empty ( $post ) ) {
		return;
	}

	// Acts only if settings are set for the post type.
	$settings = get_option( 'it_acfforms_settings' );
	if ( ! array_key_exists( $post->post_type, $settings['post-types'] ) ) {
		return;
	}
	$settings = $settings['post-types'][ $post->post_type ];

    $update = false;

	// Retrieves the value of each post field.
    foreach ( array( 'title', 'slug', 'thumbnail', 'excerpt', 'editor', 'status', 'date' ) as $post_field ) {
		// Gets the value from the plugin default field.
		if ( array_key_exists( 'it-acfforms-post-'. $post_field, $new_values ) ) {
			${$post_field} = $new_values['it-acfforms-post-'. $post_field ];
		}
		// Gets the value from the choosen field (by meta key).
		else if ( ! empty ( $settings['select-'. $post_field ] ) ) {
			$format = ( 'excerpt' === $post_field OR 'editor' == $post_field ) ? true : false;
			${$post_field} = it_acfforms_get_field_value( $settings['select-'. $post_field ], $post_id, $new_values, $format );
		}
		// Gets the value from fixed or substitution value.
		else if ( ! empty ( $settings['custom-'. $post_field ] ) ) {
			${$post_field} = $settings['custom-'. $post_field ];

			// Gets the value from posted or existing values.
			preg_match_all( '/{(.*?)}/', ${$post_field}, $keys );
			foreach ( $keys[1] as $key ) {
				// Retrieves the posted or stored value for this key.
				$value = it_acfforms_get_field_value( $key, $post_id, $new_values );

				// Acts only if a value is set and can be displayed directly.
				if ( ! empty( $value ) AND is_string( $value ) ) {
					${$post_field} = str_replace( '{'. $key .'}', $value, ${$post_field} );
				}
			}

			// Gets the value from a properties.
			preg_match_all( '/%(.*?)%/', ${$post_field}, $keys );
			foreach ( $keys[1] as $key ) {
				if ( 'user_id' === $key ) {
					$value = get_current_user_id();
				} else if ( 'display_name' === $key ) {
					$value = get_userdata( get_current_user_id() )->display_name;
				} else if ( 'date' === $key ) {
					$value = wp_date( get_option( 'date_format' ) );
				} else if ( 'post_type' === $key ) {
					$value = get_post_type_object( get_post_type( $post_id ) )->labels->singular_name;
				} else {
					//
				}

				// Acts only if a value is set and can be displayed directly.
				if ( isset( $value ) AND is_string( $value ) ) {
					${$post_field} = str_replace( '%'. $key .'%', $value, ${$post_field} );
				}
			}
		}
	}

	// Sets the post title.
	if ( isset( $title ) ) {
		$title = wp_kses_post( $title );
		if ( $post->post_title != $title ) {
			$post->post_title = $title;
			$update = true;
		}
	}
	// Rewrites the additionnal rewrite post slug setting field.
	if ( array_key_exists( 'rewrite-slug', $settings ) ) {
		$slug = $settings['rewrite-slug'];
	}
	if ( isset( $slug ) AND 1 == $slug ) {
		if ( ! isset( $title ) AND array_key_exists( '_post_title', $new_values ) ) {
			$title = wp_kses_post( $new_values['_post_title'] );
		}
		if ( isset( $title ) ) {
			$post->post_name = sanitize_title( $title );
			$update = true;
		}
	}
	// Sets the post thumbnail.
	if ( isset ( $thumbnail ) ) {
		if ( get_post_thumbnail_id( $post_id ) != $thumbnail ) {
			if ( empty( $thumbnail ) ) {
				delete_post_thumbnail( $post_id );
			} else {
				set_post_thumbnail( $post_id, $thumbnail );
			}
		}
	}
	// Sets the excerpt.
	if ( isset ( $excerpt ) ) {
		if ( $post->post_excerpt != $excerpt ) {
			$post->post_excerpt = sanitize_textarea_field( $excerpt );
			$update = true;
		}
	}
	// sets the content.
	if ( isset ( $editor ) ) {
		if ( $post->post_content != $editor ) {
			$post->post_content = wp_kses_post( $editor );
			$update = true;
		}
	}
	// Sets the publication status.
	if ( isset ( $status ) ) {
		if ( $post->post_status != $status AND array_key_exists( $status, it_acfforms_get_statuses( true ) ) ) {
			$post->post_status = $status;
			$update = true;
		}
	}

	// Deletes existing cron jobs.
	wp_clear_scheduled_hook( 'publish_future_post', array( $post_id ) );

	// Sets Post dates and cron job.
	if ( isset ( $date ) ) {
		if ( $post->post_date != $date ) {
			$_date = date_create_from_format( 'Y-m-d H:i:s', $date, wp_timezone() );
			if ( false !== $_date ) {
				$post->post_date     = $date;
				$post->post_date_gmt = gmdate( 'Y-m-d H:i:s', $_date->getTimestamp() );
				$post->edit_date     = true;
				wp_schedule_single_event( strtotime( $post->post_date_gmt ), 'publish_future_post', array( $post->ID ) );
				$update = true;
			}
		}
	}

	// Updates the post.
	if ( true === $update ) {
		wp_update_post( $post );
	}

	/**
	 * Allows plugins to act after custom data has been saved.
	 *
	 * @param int $post_id The Post ID.
	 *
	 * @since 1.0.0
	 */
	do_action( 'it_acfforms_after_save_post', $post_id );
}

/**
 * Updates the Search and Filter cache after submission with frontend Form.
 *
 * SF's cache is not automatically updated after a publication is created via
 * the public site.
 *
 * @since 1.0.0
 *
 * @link https://www.advancedcustomfields.com/resources/acf-save_post/
 * @link https://searchandfilter.com/documentation/action-filter-reference/
 *
 * @param int $post_id
 */
add_action( 'acf/save_post', 'it_acfforms_save_update_sf_cache', 20 );
function it_acfforms_save_update_sf_cache( $post_id ) {
	do_action( 'search_filter_update_post_cache', intval( $post_id ) );
}
