<?php
/*
 * Checks whether a user has access to the frontend form for level `meta`.
 *
 * In these examples, the user can edit the post if :
 * - the value of his `can-edit-post` metadata is `yes`.
 * - the value of the post meta `editable-by-all` is 1.
 *
 * Here, other types are not tested (i.e. the user won't have access).
 *
 * @param bool   $user_has_access Whether the user has access. Default false.
 * @param string $post_type       The concerned Post Type.
 * @param int    $post_id         The concerned Post ID or `new_post`.
 * @param obj    $user            The WP_User object.
 *
 * @return bool Whether the user has access.
 */
function itopie_user_has_meta_level( $user_has_access, $post_type, $post_id, $user ) {
	// Acts on all posts.
	if ( 'post' === $post_type ) {
		$usermeta = get_user_meta( $user->ID, 'can-edit-post', true );
		if ( 'yes' == $usermeta ) {
			$user_has_access = true;
		}
	}
	// Acts on posts for existing posts only (not for new_post).
	if ( 'post' === $post_type AND is_numeric( $post_id ) ) {
		$postmeta = get_post_meta( $post_id, 'editable-by-all', true );
		if ( '1' == $postmeta ) {
			$user_has_access = true;
		}
	}
	return $user_has_access;
}
add_filter( 'it_acfforms_user_has_meta_level', 'itopie_user_has_meta_level', 10, 4 );

/*
 * Checks whether a user has access to the frontend form for level `rua`.
 *
 * In this example, the right depends on the level of the "Restrict User Access"
 * extension as defined in the settings via the custom field, linked to the
 * custom access level.
 *
 * @param bool   $user_has_access Whether the user has access. Default false.
 * @param string $post_type       The concerned Post Type.
 * @param int    $post_id         The concerned Post ID or `new_post`.
 * @param obj    $user            The WP_User object.
 *
 * @return bool Whether the user has access.
 */
function itopie_user_has_rua_level( $user_has_access, $post_type, $post_id, $user ) {
	if ( ! function_exists( 'rua_get_user' ) ) {
		return $user_has_access;
	}
	$settings = get_option( 'it_acfforms_settings' );
	// Acts only on supported Post Types.
	if ( ! array_key_exists( $post_type, $settings['post-types'] ) ) {
		return $user_has_access;
	}
	$settings = $settings['post-types'][ $post_type ];
	// Tests the right via the custom settings field.
	if ( array_key_exists( 'rua', $settings ) AND ! empty ( $settings['rua'] ) ) {
		$rua_user = rua_get_user( $user );
		$user_has_access = $rua_user->has_level( $settings['rua'] );
	}
	return $user_has_access;
}
add_filter( 'it_acfforms_user_has_rua_level', 'itopie_user_has_rua_level', 10, 4 );
