<?php
/*
 * Manages plugin settings
 */
defined( 'ABSPATH' ) || exit;

/**
 * Registers the option.
 *
 * @since 1.0.0
 */
add_action( 'admin_init', 'it_acfforms_register_setting' );
function it_acfforms_register_setting() {
	register_setting( 'it_acfforms_settings', 'it_acfforms_settings', array(
		'sanitize_callback' => 'it_acfforms_save_settings',
	) );
}

/**
 * Adds settings entry to admin menu.
 *
 * @since 1.0.0
 */
add_action( 'admin_menu', 'it_acfforms_add_options_page' );
function it_acfforms_add_options_page() {
	add_options_page(
		esc_html__( 'Frontend Forms settings', 'it-acfforms' ),
		esc_html__( 'Frontend Forms', 'it-acfforms' ),
		'manage_options',
		'it_acfforms_settings',
		'it_acfforms_settings_page',
	);
}

/**
 * Adds a link to the settings in the plugin's quick links.
 *
 * @since 1.0.0
 *
 * @param array $links Existing links.
 *
 * @return array $links Links to display.
 */
add_filter( 'plugin_action_links_'. IT_ACFFORMS_BASENAME, 'it_acfforms_plugin_action_links' );
function it_acfforms_plugin_action_links( $links ) {
	$links[] = '<a href="'. esc_url( get_admin_url( null, 'options-general.php?page=it_acfforms_settings' ) ) .'">'. esc_html__( 'Settings', 'it-acfforms' ) .'</a>';
	return $links;
}

/**
 * Adds links to plugin metadata.
 *
 * @since 1.0.0
 *
 * @param string[] $meta Plugin's metadata (version, author, author and plugin URI).
 * @param string   $file Path to the plugin file relative to the plugins dir.
 *
 * @return string[] Plugin's metadata with news links.
 */
add_filter( 'plugin_row_meta', 'it_acfforms_plugin_row_meta', 10, 2 );
function it_acfforms_plugin_row_meta( $meta, $file ) {
	if ( IT_ACFFORMS_BASENAME === $file ) {
		$meta[] = '<a href="https://gitlab.inubo.ch/plugins-wordpress/itopie-frontend-forms" target="_blank">'. esc_html__( 'GIT repository', 'it-acfforms' ) .'</a>';
	}
	return $meta;
}

/**
 * Creates the metaboxes for each settings section.
 *
 * @since 1.0.0
 */
add_action( 'load-settings_page_it_acfforms_settings', 'it_acfforms_add_meta_boxes' );
function it_acfforms_add_meta_boxes() {
	add_meta_box(
		'main',
		esc_html__( 'General settings', 'it-acfforms' ),
		'it_acfforms_settings_fields',
		'settings_page_it_acfforms_settings',
		'settings',
		'default',
	);
	foreach ( it_acfforms_get_post_types() as $post_type => $label ) {
		add_meta_box(
			$post_type,
			sprintf(
				/* translators: %s: Post Type translated label */
				esc_html__( '%s settings', 'it-acfforms' ),
				$label,
			),
			'it_acfforms_settings_fields',
			'settings_page_it_acfforms_settings',
			'settings',
			'default',
		);
	}
}

/**
 * Creates tabs and links in the help section.
 *
 * @since 1.0.0
 */
add_action( 'load-settings_page_it_acfforms_settings', 'it_acfforms_settings_help' );
function it_acfforms_settings_help() {
	$screen = get_current_screen();
	$tabs = array(
		'intro'     => esc_html__( 'Using frontend forms', 'it-acfforms' ),
		'post-type' => esc_html__( 'Settings by Post Type', 'it-acfforms' ),
		'shortcode' => esc_html__( 'Shortcode', 'it-acfforms' ),
		'endpoint'  => esc_html__( 'Endpoint', 'it-acfforms' ),
		'media'     => esc_html__( 'Media library', 'it-acfforms' ),
		'filters'   => esc_html__( 'Available filters', 'it-acfforms' ),
	);
	foreach ( $tabs as $id => $label ) {
		if ( is_file( IT_ACFFORMS_PATH .'includes/admin/help/'. $id .'.php' ) ) {
			$default = it_acfforms_get_default_settings( 'post-type' );
			ob_start();
			include( IT_ACFFORMS_PATH .'includes/admin/help/'. $id .'.php' );
			$content = ob_get_clean();
			$screen->add_help_tab( array(
				'title'   => $label,
				'id'      => $id,
				'content' => $content,
			) );
		}
	}
	$links = array(
		'#0' => esc_html__( 'On Gitlab', 'it-acfforms' ),
		'#1' => esc_html__( 'About', 'it-acfforms' ),
		'#2' => esc_html__( 'FAQ', 'it-acfforms' ),
		'#3' => esc_html__( 'Developpers', 'it-acfforms' ),
	);
	$help_sidebar = '<h4>'. esc_html__( 'More information', 'it-acfforms' ) .'</h4>';
	foreach ( $links as $url => $label ) {
		$help_sidebar.= sprintf( '<p><a href="%s">%s</a></p>', esc_url( $url ), $label );
	}
	$screen->set_help_sidebar( $help_sidebar );
}

/**
 * Displays the settings page.
 *
 * @since 1.0.0
 */
function it_acfforms_settings_page() {
	?>
	<div class="wrap acf-admin-page it-acfforms-settings-wrap">
		<h1><?php echo get_admin_page_title(); ?></h1>

		<p><?php esc_html_e( 'Easily set up frontend forms with Advanced Custom Fields, for any Post Type created or not with ACF, and with your selection of ACF field groups and Post property fields.', 'it-acfforms' ); ?></p>

		<form method="post" action="options.php">

	<?php
	settings_fields( 'it_acfforms_settings' );

	// Creates nonce for metaboxes ajax-actions (toggle and order).
	wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce' );
	wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce' );

	do_meta_boxes( 'settings_page_it_acfforms_settings', 'settings', '' );

	submit_button( esc_attr__( 'Submit', 'it-acfforms') );
	?>
		</form>
	</div>
	<?php
}

/**
 * Displays the fields via ACF functions.
 *
 * @param mixed $object  Empty.
 * @param array $element {
 *     Argument passed by add_meta_box.
 *
 *     @type string $id       Metabox ID.
 *     @type string $title    Metabox title.
 *     @type string $callback Metabox callback function (this one).
 *     @type array  $args     Optional. Default empty array.
 *     @type int    $priority Optional. Is used for ordering. Default 10.
 *  }
 *
 * @since 1.0.0
 */
function it_acfforms_settings_fields( $object, $element ) {

	$settings = get_option( 'it_acfforms_settings' );
	$prefix   = 'it_acfforms_settings';
	$pre_key  = '';
	$default  = it_acfforms_get_default_settings();

	if ( array_key_exists( $element['id'], it_acfforms_get_post_types() ) ) {
		$settings = $settings['post-types'][ $element['id'] ];
		$prefix  .= '[post-types]';
		$pre_key  = $element['id'] .'][';
		$default  = $default['post-type'];
	}

	foreach ( it_acfforms_get_settings_fields( $element['id'] ) as $key => $field ) {

		// Completes the field’s arguments.
		if ( ! array_key_exists( 'type', $field ) ) {
			$field['type'] = 'text';
		}
		if ( 'message' != $field['type']  ) {
			$field = wp_parse_args(
				$field,
				array(
					'key'     => $pre_key . $key,
					'prefix'  => $prefix,
					'value'   => ( isset( $settings[ $key ] ) ? $settings[ $key ] : ( array_key_exists( $key, $default ) ? $default[ $key ] : '' ) ),
					'default' => ( array_key_exists( $key, $default ) ? $default[ $key ] : '' ),
				)
			);
		}
		// Supports hidden fields with ACF conditions.
		if ( array_key_exists( 'hidden', $field ) ) {
			$field['value'] = $field['hidden'];
			$field['wrapper']['class'] = 'it-acfforms-hidden';
		}
		// Renders the field.
		acf_render_field_wrap(
			$field,
			'div',
			( 'message' === $field['type'] ) ? 'field' : 'label',
			true
		);
	}

	submit_button( esc_attr__( 'Save all', 'it-acfforms'), 'it-acfforms-submit' );

	// Adds some classes in postboxes with JS ACF model.
	?>
	<script type="text/javascript">
	if ( typeof acf !== 'undefined' ) {
		acf.newPostbox({
			'id': '<?php echo $element['id']; ?>',
		});
	}
	</script>
	<?php
}

/**
 * Returns fields parameters for a section of settings.
 *
 * Parameters are designed to be rendered with ACF.
 *
 * @since 1.0.0
 *
 * @param string $section   Section for which to return fields. Accepts :
 *                          general or name of a supported post type.
 *
 * @return array Fields to be displayed for the section or empty array.
 */
function it_acfforms_get_settings_fields( $section ) {
	$fields = array();
	// Sets general settings fields.
	if( 'main' === $section ) {
		$fields = array(
			'shortcode-in'  => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Shortcode locations', 'it-acfforms' ),
				'instructions' => esc_html__( 'Optional. List the publication IDs where the short code is used, separated by commas.', 'it-acfforms' ),
			),
			'endpoint-link' => array(
				'type'         => 'true_false',
				'label'        => esc_html__( 'Return endpoint link instead of edit link', 'it-acfforms' ),
				'instructions' => esc_html__( 'If checked and the endpoint is active, visible edit links will lead to the frontend form.', 'it-acfforms' ),
				'ui'           => true,
			),
		);
	}
	// Sets Post Types fields.
	else {
		if ( ! array_key_exists( $section, it_acfforms_get_post_types() ) ) {
			return $fields;
		}
		$post_type = $section;
		$post_type_object = get_post_type_object( $post_type );
		$default = it_acfforms_get_default_settings( 'post-type' );
		// Sets fields common choices.
		$post_field_choices = array(
			'default'  => esc_html__( 'Insert default field', 'it-acfforms' ),
			'select'   => esc_html__( 'Choose a custom field', 'it-acfforms' ),
			'custom'   => esc_html__( 'Enter a fixed value or use substitution variables', 'it-acfforms' ),
			'none'     => esc_html__( 'Do not display or update via the form', 'it-acfforms' ),
		);
		global $wpdb;
		$post_meta_choices = $wpdb->get_col( "SELECT DISTINCT meta_key FROM $wpdb->postmeta AS meta
		LEFT JOIN $wpdb->posts as post ON post.ID = meta.post_id
		WHERE post.post_type = '$post_type'
		AND meta.meta_key NOT LIKE '\_%'
		ORDER BY meta_key" );
		$post_meta_choices = array_combine( $post_meta_choices, $post_meta_choices );

		$fields = array(
			'tab-enabling'     => array(
				'type'      => 'tab',
				'label'     => esc_html__( 'Enabling', 'it-acfforms' ),
				'endpoint'  => 1,
			),
			'shortcode'       => array(
				'type'         => 'true_false',
				'label'        => esc_html__( 'Enable shortcode', 'it-acfforms' ),
				'instructions' => esc_html__( 'The shortcode displays the creation/edition form on the site.', 'it-acfforms' ),
				'ui'           => true,
			),
			'endpoint'       => array(
				'type'         => 'true_false',
				'label'        => esc_html__( 'Enable endpoint', 'it-acfforms' ),
				'instructions' => esc_html__( 'The endpoint links to the form for updating an existing Post.', 'it-acfforms' ),
				'ui'           => true,
			),
			'tab-access'     => array(
				'type'         => 'tab',
				'label'        => esc_html__( 'Access rights', 'it-acfforms' ),
				'conditions'   => array(
					array(
						array(
							'field'    => $post_type. '][shortcode',
							'operator' => '==',
							'value'    => '1',
						),
					),
					array(
						array(
							'field'    => $post_type. '][endpoint',
							'operator' => '==',
							'value'    => '1',
						),
					),
				),
			),
			'access' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Additional access rights', 'it-acfforms' ),
				'instructions' => esc_html__( 'Frontend form access rights granted in addition to default capabilities.', 'it-acfforms' ),
				'choices'      => it_acfforms_get_levels(),
			),
			'cap' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Required capability', 'it-acfforms' ),
				'instructions' => sprintf(
					/* translators: 1: edit_posts cap for the type */
					esc_html__( 'Having this capability will allow the use of the frontend form, in addition to the default capabilities (%s).', 'it-acfforms' ),
					$post_type_object->cap->edit_posts
				),
				'choices'      => it_acfforms_get_caps(),
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][access',
					'operator' => '==',
					'value'    => 'cap',
				),
			),
			'role' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Required role', 'it-acfforms' ),
				'instructions' => esc_html__( 'Having this role will authorize use of the frontend form, in addition to the default permissions. Choose a capability is recommended rather than choosing a role.', 'it-acfforms' ),
				'choices'      => array_map( 'translate_user_role', wp_roles()->get_names() ),
				'conditions'   => array(
					'field'    => $post_type .'][access',
					'operator' => '==',
					'value'    => 'role',
				),
			),
			'more-caps' => array(
				'type' => 'checkbox',
				'label'        => esc_html__( 'Add permissions', 'it-acfforms' ),
				'instructions' => sprintf(
					/* translators: %s: Label of the field (Additional access rights) */
					esc_html__( 'Allow people meeting the selection criteria under "%s" to edit published Posts, or those of other authors.', 'it-acfforms' ),
					esc_html__( 'Additional access rights', 'it-acfforms' ),
				),
				'choices'      => array(
					'published' => esc_html__( 'Allow editing of published Posts', 'it-acfforms' ),
					'others'    => esc_html__( 'Allow editing of other authors Posts', 'it-acfforms' ),
				),
			),
			'tab-prop'     => array(
				'type'         => 'tab',
				'label'        => esc_html__( 'Publishing properties', 'it-acfforms' ),
				'conditions'   => array(
					array(
						array(
							'field'    => $post_type. '][shortcode',
							'operator' => '==',
							'value'    => '1',
						),
					),
					array(
						array(
							'field'    => $post_type. '][endpoint',
							'operator' => '==',
							'value'    => '1',
						),
					),
				),
			),
			'author' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Author of new anonymous Posts', 'it-acfforms' ),
				'instructions' => esc_html__( 'Only effective for new publications by non-logged users.', 'it-acfforms' ),
				'choices'      => it_acfforms_get_authors( $post_type_object->cap->edit_posts ),
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][access',
					'operator' => '==',
					'value'    => 'everyone',
				),
			),
			'title-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Title field', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the Post title.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-title' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom title fields', 'it-acfforms' ),
				'instructions' => esc_html__( 'Select which field contains the Post Title.', 'it-acfforms' ),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][title-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-title' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is.', 'it-acfforms' ),
				'conditions'   => array(
					'field'    => $post_type .'][title-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-title' => array(
				'type'         => 'separator',
			),
			'slug-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Permalink change field', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to decide whether or not to modify the permalink in the event of a title change. ', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'conditions'   => array(
					'field'    => $post_type .'][title-field',
					'operator' => '!=',
					'value'    => 'none',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-slug' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom permalink change field', 'it-acfforms' ),
				'instructions' => esc_html__( 'Select the field that indicates whether to modify the permalink. The returned value must be 0 or 1.', 'it-acfforms' ),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][slug-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-slug' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => sprintf(
					/* translators: %s: 0, 1 */
					esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is. Expected value: %s.', 'it-acfforms' ),
					'0, 1'
				),
				'conditions'   => array(
					'field'    => $post_type .'][slug-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'rewrite-slug' => array(
				'type'         => 'true_false',
				'label'        => esc_html__( 'Rewrite permalink when title changes', 'it-acfforms' ),
				'ui'           => true,
				'conditions'   => array(
					array(
						'field'    => $post_type .'][title-field',
						'operator' => '!=',
						'value'    => 'none',
					),
					array(
						'field'    => $post_type .'][slug-field',
						'operator' => '==',
						'value'    => 'none',
					),
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-slug' => array(
				'type'         => 'separator',
			),
			'thumbnail-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Featured Image field', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the featured image.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-thumbnail' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom Feature Image field', 'it-acfforms' ),
				'instructions' => esc_html__( 'Select the field containing the featured image.', 'it-acfforms' ),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][thumbnail-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-thumbnail' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is.', 'it-acfforms' ),
				'conditions'   => array(
					'field'    => $post_type .'][thumbnail-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'thumbnail-media' => array(
				'type'         => 'checkbox',
				'label'        => esc_html__( 'Limit the media library choice', 'it-acfforms' ),
				'choices'      => array(
					'to' => esc_html__( 'to those uploaded to the Post', 'it-acfforms' ),
					'by' => esc_html__( 'to the author’s own media', 'it-acfforms' )
				),
				'conditions'   => array(
					'field'    => $post_type .'][thumbnail-field',
					'operator' => '==',
					'value'    => 'default',
				),
				'wrapper' => array(
					'width' => '50',
					'css'   => 'border-none',
				),
			),
			'sep-thumbnail' => array(
				'type'         => 'separator',
			),
			'excerpt-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Excerpt field', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the excerpt.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-excerpt' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom Excerpt field', 'it-acfforms' ),
				'instructions' => esc_html__( 'Select the field containing the excerpt.', 'it-acfforms' ),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][excerpt-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-excerpt' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is.', 'it-acfforms' ),
				'conditions'   => array(
					'field'    => $post_type .'][excerpt-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-excerpt' => array(
				'type'         => 'separator',
			),
			'editor-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Editor', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the Post content.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-editor' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom Content field', 'it-acfforms' ),
				'instructions' => esc_html__( 'Select the field containing the Post content.', 'it-acfforms' ),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][editor-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-editor' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is.', 'it-acfforms' ),
				'conditions'   => array(
					'field'    => $post_type .'][editor-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-editor' => array(
				'type'         => 'separator',
			),
			'status-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Post status field', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the Post Status.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'future-status' => array(
				'type'         => 'true_false',
				'label'        => sprintf(
					/* translators: %s: translated `Scheduled` status */
					esc_html__( 'Display "%s" publication status', 'it-acfforms' ),
					__( 'Scheduled' ),
				),
				'ui'           => true,
				'conditions'   => array(
					array(
						'field'    => $post_type .'][status-field',
						'operator' => '==',
						'value'    => 'default',
					),
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-status' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom Publication Status field', 'it-acfforms' ),
				'instructions' => sprintf(
					/* translators: %s: status list */
					esc_html__( 'Select the field containing the Publication Status. Expected value: %s.', 'it-acfforms' ),
					implode( ', ', array_keys( it_acfforms_get_statuses( true ) ) )
				),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][status-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-status' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => sprintf(
					/* translators: %s: status list */
					esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is. Expected value: %s.', 'it-acfforms' ),
					implode( ', ', array_keys( it_acfforms_get_statuses( true ) ) )
				),
				'conditions'   => array(
					'field'    => $post_type .'][status-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'status' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'New Posts status', 'it-acfforms' ),
				'instructions' => esc_html__( 'The post status to be assigned to the creation - this does not apply to existing Posts.', 'it-acfforms' ),
				'choices'      => array(
					'draft'   => __( 'Draft' ),
					'pending' => __( 'Pending Review' ),
					'publish' => __( 'Published' ),
				),
				'conditions'   => array(
					array(
						'field'    => $post_type .'][status-field',
						'operator' => '==',
						'value'    => 'none',
					),
					array(
						'field'    => $post_type. '][shortcode',
						'operator' => '==',
						'value'    => '1',
					),
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-status' => array(
				'type'         => 'separator',
			),
			'date-field' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Publication date', 'it-acfforms' ),
				'instructions' => esc_html__( 'If displayed, this field allows to modify the publication date.', 'it-acfforms' ),
				'choices'      => $post_field_choices,
				'hide_search'  => true,
				'wrapper' => array(
					'width' => '50',
				),
			),
			'date-format' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Date and time format on form', 'it-acfforms' ),
				'instructions' => esc_html__( 'The format displayed when editing a post, which may differ from the format returned on the site, depending on the date and time settings.', 'it-acfforms' ),
				'conditions'   => array(
					'field'    => $post_type .'][date-field',
					'operator' => '==',
					'value'    => 'default',

				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'select-date' => array(
				'type'         => 'select',
				'label'        => esc_html__( 'Custom publication date field', 'it-acfforms' ),
				'instructions' => sprintf(
					esc_html__( 'Select the field containing the Publication Date and time. Expected format: %s.', 'it-acfforms' ),
					'Y-m-d H:i:s'
				),
				'choices'      => $post_meta_choices,
				'hide_search'  => false,
				'conditions'   => array(
					'field'    => $post_type .'][date-field',
					'operator' => '==',
					'value'    => 'select',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'custom-date' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Fixed or substitute value', 'it-acfforms' ),
				'instructions' => sprintf(
					esc_html__( 'Indicate a combination of meta keys between braces, substitution variables between percent signs, and other characters as is. Expected format: %s.', 'it-acfforms' ),
					'Y-m-d H:i:s'
				),
				'conditions'   => array(
					'field'    => $post_type .'][date-field',
					'operator' => '==',
					'value'    => 'custom',
				),
				'wrapper' => array(
					'width' => '50',
				),
			),
			'sep-date' => array(
				'type'         => 'separator',
			),
			'future-only' => array(
				'type'         => 'true_false',
				'label'        => esc_html__( 'Show publication date field only for scheduled publications', 'it-acfforms' ),
				'ui'           => true,
				'conditions'   => array(
					array(
						'field'    => $post_type .'][future-status',
						'operator' => '==',
						'value'    => '1',
					),
					array(
						'field'    => $post_type .'][date-field',
						'operator' => '==',
						'value'    => 'default',
					),
				),
			),
			'tab-submission'     => array(
				'type'         => 'tab',
				'label'        => esc_html__( 'After submission', 'it-acfforms' ),
				'conditions'   => array(
					array(
						array(
							'field'    => $post_type. '][shortcode',
							'operator' => '==',
							'value'    => '1',
						),
					),
					array(
						array(
							'field'    => $post_type. '][endpoint',
							'operator' => '==',
							'value'    => '1',
						),
					),
				),
			),
			'redirect' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Redirection after submission', 'it-acfforms' ),
				'instructions' => esc_html__( 'Optional. Indicate the redirection URL after the form has been sent. A default redirection is used if not provided.', 'it-acfforms' ),
			),
			'create-feedback' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Feedback displayed after creation', 'it-acfforms' ),
				'instructions' => esc_html__( 'Optional. If not provided, the default feedback will by displayed.', 'it-acfforms' ),
				'conditions'   => array(
					array(
						array(
							'field'    => $post_type .'][shortcode',
							'operator' => '==',
							'value'    => '1',
						),
					),
				),
			),
			'update-feedback' => array(
				'type'         => 'text',
				'label'        => esc_html__( 'Feedback displayed after update', 'it-acfforms' ),
				'instructions' => esc_html__( 'Optional. If not provided, the default feedback will by displayed.', 'it-acfforms' ),
			),
		);
		// Checks Post Type support to hide unsupported fields.
		foreach( array( 'title', 'thumbnail', 'excerpt', 'editor' ) as $post_field ) {
			if ( ! post_type_supports( $post_type, $post_field ) ) {
				$fields[ $post_field .'-field']['hidden'] = 'none';
			}
		}
		// Recovers WP permalinks structure to maybe hide change slug field.
		$structure = get_option( 'permalink_structure' );
		if ( ! strstr( $structure, '%postname%' ) ) {
			$fields['slug-field']['hidden'] = 'none';
		}

		foreach( array_keys( it_acfforms_get_custom_levels() ) as $level ) {
			/**
			 * Allows plugin to add fields for each custom access level.
			 *
			 * See it_acfforms_get_custom_levels.
			 *
			 * Fields whose key already exists in `$fields` array are ignored.
			 *
			 * The dynamic part of the hook name, `$level`, refers to the level
			 * key (without `level_`).
			 *
			 * @since 1.0.0
			 *
			 * @param array             ACF fields to add. Default empty array.
			 *                          See ACF function acf_validate_field for
			 *                          information on accepted args.
			 * @param string $post_type The Post Type.
			 */
			$custom_fields = (array) apply_filters( "it_acfforms_{$level}_level_settings_fields", array(), $post_type );
			if ( ! empty( $custom_fields ) ) {
				foreach( $custom_fields as $key => $field ) {
					// Does not allow rewriting of default fields.
					if ( array_key_exists( $key, $fields ) ) {
						unset( $custom_fields[ $key ] );
					}
					// Adds the conditionnal logic.
					else {
						$custom_fields[ $key ] = wp_parse_args(
							$field,
							array(
								'conditions'  => array(
									'field'    => $post_type .'][access',
									'operator' => '==',
									'value'    => 'level_'. $level,
								),
							)
						);
					}
				}
				// Inserts custom fields after the select additional right field.
				$fields = array_merge(
					array_slice( $fields, 0, 6 ),
					$custom_fields,
					array_slice( $fields, 6 )
				);
			}
		}
	}
	return $fields;
}
