<?php
/*
 * Manages access rights on frontend
 */
defined( 'ABSPATH' ) || exit;

/**
 * Map custom caps to use frontend form on supported post types.
 *
 * Each access level is associated with a custom capability, which is tested by
 * it_acfforms_user_has_cap.
 *
 * In addition, adds WP required caps, if (according to the type settings):
 * - others authors isn't allowed for all users,
 * - edit published Post isn't allowed for all users.
 *
 * When called by it_acfforms_get_form:
 * - 1st arg of $args which contains the Post ID may be "new_post"
 * - 2th arg of $args provides the type.
 *
 * Just returns what the required capabilities are as map_meta_cap does.
 *
 * @since 1.0.0
 *
 * @param array  $caps    Primitive capabilities required of the user.
 * @param string $cap     Capability being checked.
 * @param int    $user_id The user ID.
 * @param array  $args    Optional further parameters, typically object ID.
 *
 * @return array Primitive capabilities required of the user.
 */
add_filter( 'map_meta_cap', 'it_acfforms_map_meta_cap', 10, 4 );
function it_acfforms_map_meta_cap( $caps, $cap, $user_id, $args ) {

	// Acts only if $args is provided.
	if ( empty ( $args ) ) {
		return $caps;
	}

	// Gets the type by 2nd argument or by the post ID.
	if ( 'new_post' === $args[0] AND array_key_exists( 1, $args ) ) {
		$post_type = $args[1];
	} else {
		$post = get_post( (int) $args[0] );
		if ( ! $post ) {
			return $caps;
		}
		$post_type = $post->post_type;
	}

	$settings = get_option( 'it_acfforms_settings' );

	// Acts only on supported Post Types.
	if ( ! array_key_exists( $post_type, $settings['post-types'] ) ) {
		return $caps;
	}
	$settings = $settings['post-types'][ $post_type ];

	// Acts only if frontend Form is active for this post type.
	if ( 1 != $settings['active'] ) {
		return $caps;
	}

	// Sets custom required cap, according to type settings, except for admin.
	$user = get_user_by( 'id', $user_id );
	if ( empty( $user ) OR ! in_array( 'administrator', $user->roles ) ) {
		$caps = $settings['access-caps'];
	}

	// Adds capabilities required for modification according to type settings.
	if ( isset( $post ) AND is_array( $settings['more-caps'] ) ) {
		$post_type_object = get_post_type_object( $post_type );
		if ( ! in_array( 'published', $settings['more-caps'] ) and 'publish' === $post->post_status ) {
			$caps[] = $post_type_object->cap->edit_published_posts;
		}
		if ( ! in_array( 'others', $settings['more-caps'] ) AND $user_id != $post->post_author ) {
			$caps[] = $post_type_object->cap->edit_others_posts;
		}
	}
	return $caps;
}


/**
 * Gives conditionnal access rights to use the frontend form.
 *
 * Called by current_user_can function.
 * See also it_acfforms_map_meta_cap which overloads required capabilities ($caps).
 *
 * When called by it_acfforms_get_form:
 * - 1st arg of $args array which contains the Post ID may be "new_post"
 * - 2th arg of $args array provides the type.
 *
 * @since 1.0.0
 *
 * @param array $allcaps User capabilities: true/false indexed by caps.
 * @param array $caps    Required primitive caps for the requested cap.
 * @param array $args    {
 *     Capacity control arguments indexed from 0 to n.
 *
 *     @type string	Requested capability.
 *     @type int    Concerned user ID.
 *     @type mixed	Optional further parameters, typically object ID.
 * }
 *
 * @return array The capabilities of the user being tested.
 */
add_filter( 'user_has_cap', 'it_acfforms_user_has_cap', 10, 4 );
function it_acfforms_user_has_cap( $allcaps, $caps, $args, $user ) {

	// Allow those who already have all the capabilities.
	if ( empty( array_diff_key( $caps, $allcaps ) ) ) {
		return $allcaps;
	}

	// Acts only for custom caps.
	foreach( $caps as $cap ) {
		if ( strstr( $cap, 'it_acfforms_' ) ) {
			$custom = true;
			break;
		}
	}

	if ( ! isset( $custom ) ) {
		return $allcaps;
	}

	// Acts only for a given publication (post ID or new_post).
	if ( ! array_key_exists( 2, $args ) ) {
		return $allcaps;
	}
	$post_id = $args[2];

	// Gets the type by the custom 4th arg or by the post ID.
	if ( 'new_post' === $post_id AND array_key_exists( 3, $args ) ) {
		$post_type = $args[3];
	} else {
		$post_type = get_post_type( $post_id );
		if ( ! $post_type ) {
			return $allcaps;
		}
	}

	$settings = get_option( 'it_acfforms_settings' );

	// Acts only on supported Post Type.
	if ( ! array_key_exists( $post_type, $settings['post-types'] ) ) {
		return $allcaps;
	}
	$settings = $settings['post-types'][ $post_type ];

	// Acts only if frontend Form is active for this post type.
	if ( 1 != $settings['active'] ) {
		return $allcaps;
	}

	// Checks for custom caps.
	foreach( $caps as $cap ) {

		if ( ! strstr( $cap, 'it_acfforms_' ) ) {
			continue;
		}

		if ( 'it_acfforms_default' === $cap ) {
			// is not used.
		} else if ( 'it_acfforms_cap' === $cap ) {
			if ( array_key_exists( $settings['cap'], $allcaps ) ) {
				$allcaps[ $cap ] = 1;
			}
		} else if ( 'it_acfforms_role' === $cap ) {
			if ( in_array( $settings['role'], $user->roles ) ) {
				$allcaps[ $cap ] = 1;
			}
		} else if ( 'it_acfforms_loggedin' === $cap ) {
			if ( ! empty( $user->ID ) ) {
				$allcaps[ $cap ] = 1;
			}
		} else if ( 'it_acfforms_everyone' === $cap ) {
			$allcaps[ $cap ] = 1;

		} else {
			foreach( array_keys( it_acfforms_get_custom_levels() ) as $level ) {
				/**
				 * Allows plugins to check access to custom levels.
				 *
				 * See it_acfforms_get_custom_levels.
				 *
				 * The dynamic part of the hook name, `$level`, refers to the
				 * level key (without `level_`).
				 *
				 * `$cap` has the value of `it_acfforms_{$level}`.
				 *
				 * @since	1.0.0
				 *
				 * @param bool       Whether the user has access. Default false.
				 * @param string     $post_type The concerned Post Type.
				 * @param int|string $post_id   Post ID to check or 'new_post'
				 * @param WP_User    $user      The WP_User object.
				 */
				$user_has_access = apply_filters( "it_acfforms_user_has_{$level}_level", false, $post_type, $post_id, $user );
				if ( $user_has_access ) {
					$allcaps[ $cap ] = 1;
				}
			}
		}
	}
	return $allcaps;
}
