<?php
/*
 * Manages the content displayed on the site.
 */
defined( 'ABSPATH' ) || exit;

/**
 * Inserts ACF dependicies in the html header for editing by the public site.
 *
 * The acfforms query_var is set by it_acfforms_features function.
 *
 * @since 1.0.0
 */
add_action( 'wp_head', 'it_acfforms_form_header' );
function it_acfforms_form_header() {
	if ( true === get_query_var( 'it-acfforms' ) ) {
		acf_form_head();
	}
}

/**
 * Replaces the content by the ACF form (or feedback) when the endpoint is used.
 *
 * Doesn’t redisplay form after submission.
 *
 * The it_acfforms_edit query_var is set when accessing the endpoint.
 *
 * @since 1.0.0
 *
 * @param html $content Original content
 *
 * @return html	The content to display.
 */
add_filter( 'the_content', 'it_acfforms_the_content' );
function it_acfforms_the_content( $content ) {
	if ( isset( $GLOBALS['wp_query']->query_vars['it_acfforms_edit'] ) AND is_singular() AND ! array_key_exists( 'it_acfforms_id', $_GET ) ) {
		$content = it_acfforms_get_form( 'endpoint', get_post_type(), get_the_ID() );
	}
	return $content;
}


/**
 * Creates the shortcode that generates the form (once only).
 *
 * The form is returned only if it has not already been processed.
 *
 * The acfforms query_var is set by it_acfforms_features function.
 *
 * @since 1.0.0
 *
 * @param array	$atts {
 *     Attributes passed by the shortcode.
 *
 *     @type string $post_type Optional. The post type to modify.
 *                             Default current.
 *     @type int    $post_id   Optional. Publication ID, or 'new_post' to
 *                             create a new publication. Default current.
 *     @type url    $redirect  Optional. Redirection URL after submission.
 *                             Default: see it_acfforms_get_form inline comments.
 * }
 * @param sring	$content The content (should be empty).
 *
 * @return html The form, or feedback reserved for administrators.
 */
add_shortcode( 'it-acf-form', 'it_acfforms_add_shortcode' );
function it_acfforms_add_shortcode( $atts = array(), $content = '' ) {
	$atts = array_change_key_case( (array) $atts, CASE_LOWER );
	$atts = shortcode_atts( array(
		'post_type' => get_post_type(),
		'post_id'   => get_the_ID(),
		'redirect'  => '',
	), $atts );
	if ( true === get_query_var( 'it-acfforms' ) ) {
		$content = it_acfforms_get_form( 'shortcode', $atts['post_type'], $atts['post_id'], $atts['redirect'] );
	} else if ( false === get_query_var( 'it-acfforms' ) ) {
		$content = it_acfforms_get_feedback( esc_html__( 'Only one form can be displayed at a time.', 'it-acfforms' ) );
	} else {
		$content = it_acfforms_get_feedback( sprintf(
			esc_html__( 'The form cannot be displayed. Check "%s" setting.', 'it-acfforms' ),
			esc_html__( 'Shortcode locations', 'it-acfforms' )
		) );
	}
	return $content;
}

/**
 * Replaces in the frontend the edit link by the endpoint link.
 *
 * @since 1.0.0
 *
 * @param url $link	    Link to edit the Post.
 * @param int $post_id  The post ID.
 *
 * @return url The URL to edit the Post with the frontend form.
 */
add_filter( 'get_edit_post_link', 'it_acfforms_get_edit_post_link', 10, 2 );
function it_acfforms_get_edit_post_link( $link, $post_id ) {
	$settings = get_option( 'it_acfforms_settings' );
	if ( 1 == $settings['endpoint-link'] ) {
		$url = it_acfforms_get_endpoint_url( $post_id );
		if ( ! empty( $url ) ) {
			$link = $url;
		}
	}
	return $link;
}
