<?php
/*
 * Defines functions used in the admin and frontend.
 */
defined( 'ABSPATH' ) || exit;

/**
 * Returns default settings.
 *
 * The tab- and sep- have no value and are used to display reserved keys on
 * filters help.
 *
 * @since 1.0.0
 *
 * @param string $key Optional. The setting to get.
 *
 * @param array|string The setting(s).
 */
function it_acfforms_get_default_settings( $key = '' ) {
	$settings = array(
		'shortcode'     => 1,
		'endpoint'      => 1,
		'shortode-in'   => '',
		'endpoint-link' => 1,
		'post-types'  => array(),
		'post-type'   => array(
			'tab-enabling'     => '',
			'active'           => 1,
			'shortcode'        => 1,
			'endpoint'         => 1,
			'tab-access'       => '',
			'access'           => 'default',
			'cap'              => 'edit_posts',
			'role'             => 'contributor',
			'access-caps'      => array(),
			'more-caps'        => array(),
			'tab-prop'         => '',
			'author'           => min( array_keys( ( it_acfforms_get_authors( 'manage_options' ) ) ) ),
			'title-field'      => 'default',
			'select-title'     => '',
			'custom-title'     => '',
			'sep-title'        => '',
			'slug-field'       => 'none',
			'select-slug'      => '',
			'custom-slug'      => '',
			'rewrite-slug'     => 0,
			'sep-slug'         => '',
			'thumbnail-field'  => 'none',
			'select-thumbnail' => '',
			'custom-thumbnail' => '',
			'thumbnail-media'  => array( 'to', 'by' ),
			'sep-thumbnail'    => '',
			'excerpt-field'    => 'none',
			'select-excerpt'   => '',
			'cusomt-excerpt'   => '',
			'sep-excerpt'      => '',
			'editor-field'     => 'default',
			'select-editor'    => '',
			'custom-editor'    => '',
			'sep-editor'       => '',
			'status-field'     => 'none',
			'future-status'    => 0,
			'select-status'    => '',
			'custom-status'    => '',
			'status'           => 'draft',
			'sep-status'       => '',
			'date-field'       => 'none',
			'date-format'      => 'd/m/Y H:i',
			'select-date'      => '',
			'custom-date'      => '',
			'sep-date'         => '',
			'future-only'      => 0,
			'tab-submission'   => '',
			'redirect'         => '',
			/* translators: translated Post Status */
			'create-feedback'  => esc_html__( 'The Post has been created (status: %post_status%).', 'it-acfforms' ),
			/* translators: translated Post Status */
			'update-feedback'  => esc_html__( 'The Post has been updated (status: %post_status%).', 'it-acfforms' ),
		),
	);
	if ( ! empty( $key ) AND array_key_exists( $key, $settings ) ) {
		return $settings[ $key ];
	}
	return $settings;
}

/**
 * Checks the option and creates it if missing.
 *
 * @since 1.0.0
 */
function it_acfforms_get_settings() {
	$settings = get_option( 'it_acfforms_settings' );
	if ( empty( $settings ) ) {
		update_option(
			'it_acfforms_settings',
			it_acfforms_get_default_settings()
		);
	}
	return $settings;
}

/**
 * Adds hidden entries in settings when saving them:
 *
 * - A feature is set as globally active = 1 if it’s active for a type, else 0.
 * - The frontend is set active = 1 for the type if one feature is active, else 0.
 * - Sets capabilities for verifying frontend access rights.
 *
 * Also effective when update_option is run (see post-type.php)
 *
 * @since 1.0.0
 *
 * @param array	$settings Settings sent
 *
 * @return array The settings with hidden entries.
 */
function it_acfforms_save_settings( $settings ) {
	$settings['shortcode'] = 0;
	$settings['endpoint'] = 0;
	foreach( $settings['post-types'] as $post_type => $type_settings ) {
		if ( 1 == $type_settings['shortcode'] OR 1 == $type_settings['endpoint'] ) {
			$settings['post-types'][ $post_type ]['active'] = 1;

			if ( 1 == $type_settings['shortcode'] ) {
				$settings['shortcode'] = 1;
			}
			if ( 1 == $type_settings['endpoint'] ) {
				$settings['endpoint'] = 1;
			}

			// Defines the frontend access rights.
			$custom_caps = array();
			if ( 'default' === $type_settings['access'] ) {
				$custom_caps[] = get_post_type_object( $post_type )->cap->edit_posts;
			} else {
				$custom_caps[] = 'it_acfforms_'. $type_settings['access'];
			}
			$settings['post-types'][ $post_type ]['access-caps'] = $custom_caps;
		} else {
			$settings['post-types'][ $post_type ]['active'] = 0;
		}
	}
	return $settings;
}

/**
 * Returns public Post Types except from excluded.
 *
 * The filter in it_acfforms_get_excluded_types allows to exclude others types.
 *
 * @since 1.0.0
 *
 * @return array {
 *     The post types
 *
 *     @type string     Post Type Name
 *     @type string|obj Post Type label or Post Type object
 * 	}
 */
function it_acfforms_get_post_types() {
	$args = array(
		'public' => true,
	);
	$post_types = get_post_types( $args, 'objects' );
	$post_types = array_column( $post_types, 'label', 'name' );
	$post_types = array_diff_key( $post_types, array_flip( it_acfforms_get_excluded_types() ) );
	return $post_types;
}

/**
 * Returns Post Types for which the frontend form cannot be used.
 *
 * By default, page, attachment and product types are excluded.
 *
 * @since 1.0.0
 *
 * @return array Excluded Post Types names.
 */
function it_acfforms_get_excluded_types() {
	/**
	 * Allows plugins to modify excluded Post types.
	 *
	 * @since 1.0.0
	 *
	 * @param array $exclude Excluded Post Types names.
	 */
	$excluded = (array) apply_filters( 'it_acfforms_excluded_types', array( 'page', 'attachment', 'product' ) );
	return $excluded;
}

/**
 * Returns level access labels indexed by names.
 *
 * Allows plugins to add custom access levels.
 *
 * @since 1.0.0
 *
 * @return array $args Each level label indexed by name.
 */
function it_acfforms_get_levels() {
	$levels = array(
		'default'   => esc_attr__( 'According to default capabilities', 'it-acfforms' ),
		'cap'       => esc_attr__( 'According to a custom capability', 'it-acfforms' ),
		'role'      => esc_attr__( 'According to a role', 'it-acfforms' ),
		'loggedin'  => esc_attr__( 'Anyone who is logged in', 'it-acfforms' ),
		'everyone'  => esc_attr__( 'Everyone', 'it-acfforms' ),
	);
	$custom_levels = it_acfforms_get_custom_levels();
	if ( ! empty( $custom_levels ) ) {
		foreach( $custom_levels as $key => $value ) {
			$levels[ 'level_'. $key ] = ( ! empty( $value ) AND is_string( $value ) ) ? $value : sprintf( esc_html__( 'Access level %s', 'it-acfforms' ), $key );
		}
	}
	return $levels;
}

/**
 * Returns custom access levels.
 *
 * @since 1.0.0
 *
 * @return array Custom access levels.
 */
function it_acfforms_get_custom_levels() {
	/**
	 * Allows plugins to add access levels.
	 *
	 * Access levels will be added as choices in the Post Types settings.
	 * Access rights should be checked using it_acfforms_user_has_custom_{$level}_access
	 * filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array  Custom access levels. Default empty array.
	 */
	$levels = (array) apply_filters( 'it_acfforms_custom_levels', array() );
	return $levels;
}

/**
 * Returns all capacities registered on the site.
 *
 * @since 1.0.0
 *
 * @return array The caps (as keys and values).
 */
function it_acfforms_get_caps() {
	$caps = array();
	foreach( wp_roles()->role_objects as $role ) {
		$caps = array_merge( $caps, array_keys( $role->capabilities ) );
	}
	$caps = array_unique( $caps );
	asort( $caps );
	$caps = array_combine( $caps, $caps);
	return $caps;
}

/**
 * Returns users with the requested capabilities (eg. edit_posts).
 *
 * @since 1.0.0
 *
 * @param string|string[] $caps Capabilities required.
 *
 * @return array User names indexed by ID.
 */
function it_acfforms_get_authors( $caps ) {
	$authors = array();
	$users = get_users( array(
		'capability'  => $caps,
		'orderby'     => 'display_name',
		'order'       => 'ASC',
		'fields'      => array( 'ID', 'display_name' ),
	) );
	if ( ! empty( $users ) ) {
		$authors = array_combine( array_column( $users, 'ID' ), array_column( $users, 'display_name' ) );
	}
	return $authors;
}

/**
 * Returns allowed frontend statuses.
 *
 * @since 1.0.0
 *
 * @param bool $future Whether to display "Scheduled" status.
 *
 * @return array The statuses (as keys and values).
 */
function it_acfforms_get_statuses( $future = false ) {
	$statuses = array(
		'draft'   => __( 'Draft' ),
		'pending' => __( 'Pending Review' ),
		'publish' => __( 'Published' ),
	);
	if ( true === $future ) {
		$statuses['future'] = __( 'Scheduled' );
	}
	return $statuses;
}

/**
 * Generates the ACF form for editing content from the public site.
 *
 * Checks the requested feature, type and access rights to decide whether to
 * display the form.
 *
 * The form is displayed even if no field group is active, but not if title and
 * content are also disabled.
 *
 * @since	1.0.0
 *
 * @param string $feature   The functionality used to display the form.
 *                          Accept: shortcode or endpoint.
 * @param string $post_type Post Type
 * @param string $post_id   Post ID
 * @param string $redirect  Optional. Redirection URL after submission.
 *
 * @link https://www.advancedcustomfields.com/resources/acf_form/
 *
 * @return html	The form or a feed-back (mostly only displayed to admins).
 */
function it_acfforms_get_form( $feature, $post_type, $post_id, $redirect = '' ) {

	// Checks if the required arguments are set.
	if ( empty ( $post_type ) OR empty ( $post_id ) ) {
		return it_acfforms_get_feedback( esc_html__( 'The form cannot be displayed because the required arguments are missing.', 'it-acfforms' ) );
	}

	// Checks that the post type exists.
	$post_type_object = get_post_type_object( $post_type );
	if ( empty ( $post_type_object ) ) {
		return it_acfforms_get_feedback( sprintf(
			/* translators: %s: Post Type */
			esc_html__( 'The form cannot be displayed because the type (%s) was not found.', 'it-acfforms' ),
			$post_type
		) );
	}

	// Checks that the Post Types is supported.
	$settings = get_option( 'it_acfforms_settings' );
	if ( ! array_key_exists( $post_type, $settings['post-types'] ) ) {
		return it_acfforms_get_feedback( sprintf(
			/* translators: %s: post type name */
			esc_html__( 'The frontend form cannot be used with this type (%s) because the settings are missing, usually because the type is not supported - check excluded types.', 'it-acfforms' ),
			$post_type
		) );
	}
	$settings = $settings['post-types'][ $post_type ];

	// Checks that the feature exists.
	if ( ! array_key_exists( $feature, $settings ) ) {
		return it_acfforms_get_feedback( sprintf(
			/* translators: %s: feature passed by args */
			esc_html__( 'The form cannot be displayed because this feature (%s) doesn’t exist.', 'it-acfforms' ),
			$feature
		) );
	}

	// Checks that the feature is active.
	if ( 1 != $settings[ $feature ] ) {
		return it_acfforms_get_feedback( sprintf(
			/* translators: 1: Post Type name 2: feature (endpoint or shortcode) */
			esc_html__( 'The form cannot be displayed because this feature (%1$s) is disabled for this type (%2$s).', 'it-acfforms' ),
			$feature,
			$post_type
		) );
	}

	// Check access rights.
	if ( ! current_user_can( $post_type_object->cap->edit_posts, $post_id, $post_type ) ) {
		return it_acfforms_get_feedback( esc_html__( 'You are not allowed to use this form.', 'it-acfforms' ), true );
	}

	// Defines the creation of a publication.
	$new_post = false;
	if( 'new_post' === $post_id ) {
		$user_id = get_current_user_id();
		if ( empty ( $user_id ) ) {
			$user = get_user_by( 'id', $settings['author'] );
			if ( ! empty ( $user ) ) {
				$user_id = $user->ID;
			} else {
				$user_id = it_acfforms_get_default_settings( 'post-type' )['author'];
			}
		}
		$new_post = array(
            'post_type'   => $post_type,
            'post_status' => ( array_key_exists( 'status', $settings ) ) ? $settings['status'] : it_acfforms_get_default_settings( 'post-type' )['status'],
            'post_author' => $user_id,
        );
	}

	// Defines the display of Post fields supported by ACF.
	if ( 'default' == $settings['title-field'] ) {
		$post_title = true;
	} else {
		$post_title = false;
	}
	if ( 'default' == $settings['editor-field'] ) {
		$post_content = true;
	} else {
		$post_content = false;
	}

	// Adds optional Post fields as local fields.
	it_acfforms_add_local_fields( $post_type, $post_id, $settings );

	// Retrieves ACF field groups for this Post.
	$field_groups = acf_get_field_groups(
		array(
			'post_id'   => $post_id,
			'post_type' => $post_type,
		)
	);
	// Unsets disabeld on frontend fields (considered as active if not set).
	if ( ! empty ( $field_groups ) ) {
		foreach( $field_groups as $i => $field_group ) {
			if ( array_key_exists( 'it_acfforms_active', $field_group ) AND 1 != $field_group['it_acfforms_active'] ) {
				unset( $field_groups[ $i ] );
			}
		}
	}

	// Exits if there are no fields to display.
	if ( empty( $field_groups ) AND ! $post_title AND ! $post_content ) {
		return it_acfforms_get_feedback( esc_html__( 'No fields are defined for this content type. To display the form, activate at least one ACF fields group or one Post field.', 'it-acfforms' ) );
	}

	/*
	 * Defines the redirection link after submission.
	 *
	 * @see it_acfforms_404_redirect: template_redirect is used to redirect to the
	 * archive page if it exists, or to the home page if the user does not have
	 * permission to view the publication.
	 */
	if ( empty( $redirect ) ) {
		if ( ! empty( $settings['redirect'] ) ) {
			$redirect = $settings['redirect'];
		} else {
			$redirect = '%post_url%';
		}
	}
	$redirect = add_query_arg( array(
		'it_acfforms'    => 1, // 1st arg unnecessary, to allow & on arg.
		'it_acfforms_id' => '%post_id%',
		'new'         => ( $new_post ? 1 : 0 )
	), $redirect );

	// Sets the send button.
	$html_submit_button = '
	<div class="it-acfforms-submit">
		<input type="submit" value="'. esc_attr__( 'Submit', 'it-acfforms' ) .'" class="wp-element-button">
	</div>';

	// Defines form parameters.
	$args = array(
		'id'                 => 'it-acfforms-'. $post_type .'-'. $post_id,
		'post_id'            => $post_id,
		'new_post'           => $new_post,
		'post_title'         => $post_title,
		'post_content'       => $post_content,
		'field_groups'       => array_column( $field_groups, 'key' ),
		'updated_message'    => false,
		'html_submit_button' => $html_submit_button,
		'return'             => $redirect,
	);

	// Gets the form with ACF.
	ob_start();
	acf_form( $args );
	$content = ob_get_clean();

	// Prevents the form being displayed more than once.
	set_query_var( 'it-acfforms', false );

	return $content;
}

/**
 * Displays a feedback.
 *
 * @since 1.0.0
 *
 * @param bool $to_all If everybody can see it. Default false.
 *
 * @return html The formatted feedback or nothing.
 */
function it_acfforms_get_feedback( $message, $to_all = false ) {
	if ( ! $to_all ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		} else {
			$message = sprintf( esc_html__( 'Message to admins: %s', 'it-acfforms'), $message );
		}
	}
	return '<div class="it-acfforms-feedback">'. $message .'</div>';
}

/**
 * Returns the editing link via the endpoint.
 *
 * Checks whether endpoint is active globally and for the type.
 *
 * @since 1.0.0
 *
 * @param int $post_id	Optional. The Post ID. Default: current Post.
 *
 * @return URL The edit link or an empty string.
 */
function it_acfforms_get_endpoint_url( $post_id = 0 ) {

	if ( empty ( $post_id ) ) {
		$post_id = get_the_ID();
	}
	$url = '';

	// Exits if endpoint editing is inactive.
	$settings = get_option( 'it_acfforms_settings' );
	if ( 1 != $settings['endpoint'] ) {
		return $url;
	}
	$post_type = get_post_type( $post_id );

	// Acts only on supported Post Type.
	if ( ! array_key_exists( $post_type, $settings['post-types'] ) ) {
		return $url;
	}

	// Exits if the endpoint is inactive for the publication type.
	if ( 1 != $settings['post-types'][ $post_type ]['endpoint'] ) {
		return $url;
	}

	$permalink = get_permalink( $post_id );

	// Manages link to unpublished/published Post.
	if ( 'publish' !== get_post_status( $post_id ) ) {
		$url = add_query_arg( array(
			'it_acfforms_edit' => '',
		), $permalink );
	} else {
		$url = trailingslashit( $permalink ) .'it_acfforms_edit';
	}
	return $url;
}

/**
 * Registers local fields groups to support Post fields.
 *
 * The field group is only active when the form is displayed, not when it’s saved.
 *
 * @since 1.0.0
 *
 * @param string $post_type
 * @param int    $post_id
 * @param array  $settings Settings for the Post Type.
 */
function it_acfforms_add_local_fields( $post_type, $post_id, $settings ) {

	acf_add_local_field_group( array(
		'key'        => 'it-acfforms-post-fields-group',
		'fields'     => array(),
		'location'   => array(
			array(
				array(
					'param'    => 'post_type',
					'operator' => '==',
					'value'    => $post_type,
				),
			),
		),
		'menu_order' => 10,
	) );

	// Is hidden, used to get post type in ACF acf/load_field filters.
	acf_add_local_field( array(
		'key'     => 'it-acfforms-post-type',
		'label'   => esc_html__( 'Post Type', 'it-acfforms' ),
		'name'    => 'it-acfforms-post-type',
		'type'    => 'text',
		'value'   => $post_type,
		'wrapper' => array(
			'class' => 'it-acfforms-local-field it-acfforms-hidden',
		),
		'parent'  => 'it-acfforms-post-fields-group',
	) );

	if ( 'default' === $settings[ 'slug-field' ] ) {
		acf_add_local_field( array(
			'key'     => 'it-acfforms-post-slug',
			'label'   => apply_filters( 'it_acfforms_slug_label', esc_html__( 'The link to this publication must match its title', 'it-acfforms' ), $post_type ),
			'name'    => 'it-acfforms-post-slug',
			'type'    => 'true_false',
			'value'   => ( is_numeric( $post_id ) ) ? get_post_meta( $post_id, 'it-acfforms-post-slug', true ) : 0,
			'ui'      => true,
			'wrapper' => array(
				'class' => 'it-acfforms-local-field',
			),
			'parent'  => 'it-acfforms-post-fields-group',
		) );
	}
	if ( 'default' === $settings[ 'thumbnail-field' ] ) {
		$field =  array(
			'key'                  => 'it-acfforms-post-thumbnail',
			'label'                => apply_filters( 'it_acfforms_thumbnail_label', esc_html__( 'Featured Image' ), $post_type ),
			'name'                 => 'it-acfforms-post-thumbnail',
			'type'                 => 'image',
			'value'                => ( is_numeric( $post_id ) ) ? get_post_thumbnail_id( $post_id ) : '',
			'return_format'        => 'id',
			'wrapper'              => array(
				'class' => 'it-acfforms-local-field',
			),
			'parent'               => 'it-acfforms-post-fields-group',
		);
		if ( array_key_exists( 'thumbnail-media', $settings ) ) {
			if ( in_array( 'to', $settings['thumbnail-media'] ) ) {
				$field['library'] = 'uploadedTo';
			}
			if ( in_array( 'by', $settings['thumbnail-media'] ) ) {
				$field['it_acfforms_limit_media'] = '1';
			}
		}
		acf_add_local_field( $field );
	}
	if ( 'default' === $settings[ 'excerpt-field' ] ) {
		acf_add_local_field( array(
			'key'     => 'it-acfforms-post-excerpt',
			'label'   => apply_filters( 'it_acfforms_excerpt_label', esc_html__( 'Excerpt', 'it-acfforms' ), $post_type ),
			'name'    => 'it-acfforms-post-excerpt',
			'type'    => 'textarea',
			'value'   => ( is_numeric( $post_id ) ) ? get_post_field( 'post_excerpt', $post_id ) : '',
			'wrapper' => array(
				'class' => 'it-acfforms-local-field',
			),
			'parent'  => 'it-acfforms-post-fields-group',
		) );
	}
	if ( 'default' === $settings[ 'status-field' ] ) {
		$future = false;
		if ( array_key_exists( 'future-status', $settings ) AND 1 == $settings['future-status'] ) {
			$future = true;
		}
		acf_add_local_field( array(
			'key'           => 'it-acfforms-post-status',
			'label'         => apply_filters( 'it_acfforms_status_label', esc_html__( 'Publication status', 'it-acfforms' ), $post_type ),
			'name'          => 'it-acfforms-post-status',
			'type'          => 'select',
			'choices'       => it_acfforms_get_statuses( $future ),
			'value'         => ( is_numeric( $post_id ) ) ? get_post_field( 'post_status', $post_id ) : '',
			'wrapper' => array(
				'class' => 'it-acfforms-local-field',
			),
			'parent'  => 'it-acfforms-post-fields-group',
		) );
	}
	if ( 'default' === $settings[ 'date-field' ] ) {
		$field = array(
			'key'           => 'it-acfforms-post-date',
			'label'         => apply_filters( 'it_acfforms_date_label', esc_html__( 'Publication date', 'it-acfforms' ), $post_type ),
			'name'          => 'it-acfforms-post-date',
			'type'          => 'date_time_picker',
			'display_format' => ( ! empty ( $settings['date-format'] ) ) ? $settings['date-format'] : it_acfforms_get_default_settings( 'post-type' )['date-format'],
			'return_format' => get_option( 'date_format' ) .' '. get_option( 'time_format' ),
			'first_day'     => get_option( 'start_of_week' ),
			'value'         => ( is_numeric( $post_id ) ) ? get_post_field( 'post_date', $post_id ) : '',
			'wrapper' => array(
				'class' => 'it-acfforms-local-field',
			),
			'parent'  => 'it-acfforms-post-fields-group',
		);
		if ( array_key_exists( 'future-only', $settings ) AND 1 == $settings['future-only'] ) {
			$field['conditions'] = array(
				'field'    => 'it-acfforms-post-status',
				'operator' => '==',
				'value'    => 'future',
			);
		}
		acf_add_local_field( $field );
	}
}

/**
 * Returns the posted or stored value for a key.
 *
 * - Gets the ACF field to retreive posted value by the field key.
 *   /!\ get_field_object ACF function doesn't work for new posts.
 * - Gets the ACF formatted submitted value, if sent for this key.
 * - Else gets the ACF non submitted value, if it exists for this key.
 * - Else gets the metadata value.
 *
 * @since 1.0.0
 *
 * @param string $key            Meta key
 * @param int    $post_id        Post ID
 * @param array  $posted_values  Values posted by the form ($_POST['acf']).
 * @param bool   $format         Whether or not to format the value with ACF.
 */
function it_acfforms_get_field_value( $key, $post_id, $posted_values, $format = false ) {
	$value = '';
	// Gets the ACF field.
	$field = acf_get_field( $key, $post_id );
	if ( $field ) {
		// Gets the ACF formatted submitted value.
		if ( array_key_exists( $field['key'], $posted_values ) ) {
			$value = $posted_values[ $field['key'] ];
		} else {
			// Gets the ACF non submitted value.
			if ( $_field = get_field_object( $field['key'], $post_id ) ) {
				$value = $_field['value'];
			}
		}
		if ( true === $format ) {
			$value = acf_format_value( $value, $post_id, $field );
		}
	} else {
		// Gets the metadata value.
		$value = get_post_meta( $post_id, $key, true );
	}
	return $value;
}
