<?php
/*
 * Manages query requests.
 */

defined( 'ABSPATH' ) || exit;

/**
 * Adds a query variable to check whether frontend editing is requested.
 *
 * Used to load ACF headers and to display the form conditionally.
 *
 * @since 1.0.0
 *
 * @param array $query_vars WP query_vars entries.
 *
 * @return array WP query_vars entries.
 */
add_filter( 'query_vars', 'it_acfforms_query_vars' );
function it_acfforms_query_vars( $query_vars ) {
	$query_vars[] = 'it-acfforms';
	return $query_vars;
}

/**
 * Adds the endpoint for modifying a given publication.
 *
 * The feature can be disabled by Post Type in the settings.
 *
 * @since 1.0.0
 */
add_action( 'init', 'it_acfforms_add_endpoint' );
function it_acfforms_add_endpoint() {
	$settings = get_option( 'it_acfforms_settings' );
	if ( 1 == $settings['endpoint'] ) {
	    add_rewrite_endpoint( 'it_acfforms_edit', EP_PERMALINK );

	    if ( empty( get_option( 'it_acfforms_permalinks_flushed' ) ) ) {
			flush_rewrite_rules( false );
			update_option( 'it_acfforms_permalinks_flushed', 1 );
		}
	}
}

/**
 * Sets it-acfforms query_var to true when using the frontend form.
 *
 * If the shortcode is enabled, to avoid loading dependencies site-wide, a
 * setting (shortcode-in) can be set to contain the publication IDs on which
 * they are to be loaded.
 * - if not set, the dependencies are loaded site-wide.
 * - if set, the shortcode only works on these publications.
 *
 * @since 1.0.0
 *
 * @param WP $wp Current WordPress environment instance (passed by reference).
 */
add_action( 'wp', 'it_acfforms_features' );
function it_acfforms_features( $wp ) {
	$settings = get_option( 'it_acfforms_settings' );
	if ( 1 == $settings['endpoint'] ) {
		if ( isset( $GLOBALS['wp_query']->query_vars['it_acfforms_edit'] ) AND is_singular() ) {
			set_query_var( 'it-acfforms', true );
		}
	}
	if ( 1 == $settings['shortcode'] ) {
		if ( empty( $settings['shortcode-in'] ) OR in_array( get_the_ID(), explode( ',', $settings['shortcode-in'] ) ) ) {
			set_query_var( 'it-acfforms', true );
		}
	}
}

/**
 * Redirects to the archive or home page after submission if 404.
 *
 * Usefull if the publication is not published and default cap is missing.
 *
 * @since 1.0.0
 */
add_action( 'template_redirect', 'it_acfforms_404_redirect', 5 );
function it_acfforms_404_redirect() {
	if ( is_404() AND array_key_exists( 'it_acfforms_id', $_GET ) ) {
		$post_id = (int) $_GET['it_acfforms_id'];
		$post_type = get_post_type( $post_id );
		$post_type_object = get_post_type_object( $post_type );
		if ( ! current_user_can( $post_type_object->cap->edit_posts ) ) {
			if ( 1 == $post_type_object->has_archive ) {
				$redirect = get_post_type_archive_link( $post_type );
			} else {
				$redirect = get_home_url();
			}
			$redirect = add_query_arg( array(
				'it_acfforms_id' => $_GET['it_acfforms_id'],
				'new'            => $_GET['new']
			), $redirect );
			wp_redirect( $redirect );
		}
	}
}

/**
 * Handles redirection (after submission) if header has been sent.
 * 
 * ACF class acf_form_front provides a redirect after submission, but
 * depending on the theme, the header has already been sent when the 
 * form is displayed, in which case wp_redirect won't work.
 * 
 * @since 1.0.3
 */
add_filter( 'wp_redirect', 'it_acforms_redirect', 10, 2 );
function it_acforms_redirect( $location, $status ) {
	if ( headers_sent() ) {
		echo '<script>location.href = "'. $location .'"</script>';
		// Avoids blank page if JS is disabled.
		$location = false;
	}
	return $location;
}
