<?php
/*
 * Plugin Name: Front Forms with ACF
 * Description: Easily set up frontend forms with Advanced Custom Fields, for any Post Type created or not with ACF, and with your selection of ACF field groups and Post property fields.
 * Version: 1.0.4
 * Requires at least: 4.7
 * Author: itopie informatique
 * Author URI: https://www.itopie.ch
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: it-acfforms
 * Domain Path: /languages
 */

defined( 'ABSPATH' ) || exit;

/**
 * Defines the path to the plugin's main file.
 *
 * @since 1.0.0
 */
if ( ! defined( 'IT_ACFFORMS_FILE' ) ) {
	define( 'IT_ACFFORMS_FILE', __FILE__ );
}

/**
 * Defines the plugin’s url, used to include CSS and JS files.
 *
 * @since 1.0.0
 *
 * @var	string
 */
if ( ! defined( 'IT_ACFFORMS_URL' ) ) {
	define( 'IT_ACFFORMS_URL', plugin_dir_url( IT_ACFFORMS_FILE ) );
}

/**
 * Defines the plugin path in the file system, used to include dependencies.
 *
 * @since 1.0.0
 *
 * @var string
 */
if ( ! defined( 'IT_ACFFORMS_PATH' ) ) {
	define( 'IT_ACFFORMS_PATH', plugin_dir_path( IT_ACFFORMS_FILE ) );
}

/**
 * Defines the plugin name used for plugin links in admin.
 *
 * @since 1.0.0
 *
 * @var	string
 */
if ( ! defined( 'IT_ACFFORMS_BASENAME' ) ) {
	define( 'IT_ACFFORMS_BASENAME', plugin_basename( IT_ACFFORMS_FILE ) );
}

/**
 * Defines the plugin path relative to the plugin directory, used to load translation files.
 *
 * @since 1.0.0
 *
 * @var	string
 */
if ( ! defined( 'IT_ACFFORMS_BASEDIR' ) ) {
	define( 'IT_ACFFORMS_BASEDIR', dirname( IT_ACFFORMS_BASENAME ) );
}

/**
 * Loads the plugin’s functionalities.
 *
 * @since 1.0.0
 */
add_action( 'plugins_loaded', 'it_acfforms_load' );
function it_acfforms_load() {

	/**
	 * Loads the plugin’s translated strings.
	 *
	 * @since 1.0.0
	 */
	load_plugin_textdomain( 'it-acfforms', false, IT_ACFFORMS_BASEDIR . '/languages/' );

	// Loads if ACF is present, otherwise deactivates.
	if ( function_exists( 'get_field' ) ) {

		/**
		 * Defines functions used in the admin and frontend.
		 *
		 * @since 1.0.0
		 */
		require_once( IT_ACFFORMS_PATH . 'includes/functions.php' );

		// Checks the option and creates it if missing.
		it_acfforms_get_settings();

		/**
		 * Manages Post Types settings and row actions.
		 *
		 * @since 1.0.0
		 */
		require_once( IT_ACFFORMS_PATH . 'includes/post-type.php' );

		/**
		 * Customizes frontend forms and corrects media queries.
		 *
		 * @since 1.0.0
		 */
		require_once( IT_ACFFORMS_PATH . 'includes/acf.php' );

		/**
		 * Manages query requests.
		 *
		 * Must also be loaded in admin so that the endpoint is known when
		 * permalinks settings are saved.
		 *
		 * @since 1.0.0
		 */
		require_once( IT_ACFFORMS_PATH . 'includes/queries.php' );

		if ( is_admin() ) {

			/**
			 * Inserts JS and CSS files.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/admin/assets.php' );

			/**
			 * Manages plugin settings.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/admin/settings.php' );

			/**
			 * Adds tabs and options to ACF admin interfaces.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/admin/acf-settings.php' );
		} else {

			/**
			 * Inserts JS and CSS files.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/assets.php' );

			/**
			 * Manages access rights on frontend.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/caps.php' );

			/**
			 * Manages the content displayed on the site.
			 *
			 * @since 1.0.0
			 */
			require_once( IT_ACFFORMS_PATH . 'includes/content.php' );
		}
	}
	// Displays a notification if ACF is missing.
	else {
		add_action( 'admin_notices', 'it_acfforms_requirements_notice', 0 );
		add_action( 'admin_init', 'it_acfforms_deactivates', 0 );
	}
}

/**
 * Creates options on plugin activation.
 *
 * @since 1.0.0
 */
register_activation_hook( IT_ACFFORMS_FILE, 'it_acfforms_activate' );
function it_acfforms_activate() {
	if ( ! current_user_can( 'activate_plugins' ) ) {
        return;
	}

	/**
	 * Defines functions used in the admin and frontend.
	 *
	 * @since 1.0.0
	 */
	require_once( IT_ACFFORMS_PATH . 'includes/functions.php' );

	// Checks the option and creates it if missing
	it_acfforms_get_settings();

	// Enables permalink rules to be rewritten after activation.
	update_option( 'it_acfforms_permalinks_flushed', 0 );
}

/**
 * Removes options when the plugin is deleted.
 *
 * @since 1.0.0
 */
register_uninstall_hook( IT_ACFFORMS_FILE, 'it_acfforms_uninstall' );
function it_acfforms_uninstall() {
	if ( ! current_user_can( 'activate_plugins' ) ) {
        return;
	}
	// Efface les options.
	delete_option( 'it_acfforms_settings' );
}

/**
 * Displays a notification if ACF is missing.
 *
 * @since 1.0.0
 */
function it_acfforms_requirements_notice() {
	echo '<div class="notice notice-error">
	<p>'. 	sprintf(
		/* translators: 1: Plugin name, 2: Advanced Custom Field with link */
		esc_html__( 'The extension "%1$s" has been deactivated as it only works if the %2$s extension (or the pro version) is active.', 'it-acfforms' ),
		__( 'Front Forms with ACF', 'it-acfforms' ),
		'<a href="https://wordpress.org/plugins/advanced-custom-fields/" target="_blank">Advanced Custom Fields</a>'
	) .'</p>
    </div>';
}

/**
 * Deactivates itself.
 * 
 * @since 1.0.1
 */
function it_acfforms_deactivates() {
	deactivate_plugins( IT_ACFFORMS_FILE );
}
